C*****************************************
       program multislice stereology_1
C*****************************************
       implicit none
	   include 'common.fi'

       !parameter (CD=9)
       !parameter (CD2=9)
       !dimension gbd(CD,CD,CD,CD2,4*CD2)
	   dimension gbd(12,12,12,12,48)
       dimension e1(3), e2(3)
       dimension gg1(3,3), gg2(3,3), dg(3,3)
       dimension so1(3,3), so2(3,3), sa(2), so2t(3,3)

       write(6,*) '=============================================='
	   write(6,*) ' PROGRAM TO CALCULATE THE FIVE PARAMETER'
	   write(6,*) ' GRAIN BOUNDARY CHARACTER DISTRIBUTION '
	   write(6,*) ' FROM THE STEREOLOGICAL ANALYSIS OF GRAIN'
	   write(6,*) ' BOUNDARY TRACES OBSERVED ON MULTIPLE SECTION '
	   write(6,*) ' PLANES.  THE PROGRAM USES THE MEAN FIELD '
	   write(6,*) ' STEREOLOGICAL PROCEEDURE DESCRIBED BY ,'
	   write(6,*) ' SAYLOR ET AL.MET TRANS. 35A (2004) 1981,  '
	   write(6,*) ' MODIFIED TO ACCOUNT FOR PREFERRED  '
	   write(6,*) ' ORIENTATIONS AND MULTIPLE SECTION PLANES'
	   write(6,*)	   
	   write(6,*) '  version 05/27/2011'
       write(6,*) '  rohrer@cmu.edu'	   
       write(6,*) '=============================================='
       
	   version = 'version 05/27/11'
	   
       pi = 4.0*atan(1.0)
	   eps = 1.e-6
	   HCD = 90.0
	   t_out=0

c  The section reads information from the input file, including the data file name,
c  the symmetry type.
 5     format(A)
       open(21, file='input.txt', status='old')
	   read(21,*)
	   read(21,*) keyword
	   read(21,*) tempword, tempword, tempword
c      read(21,*)
       read(21,*) msym, rotation, radian
	   read(21,*) tempword   
	   read(21,*) CD, CD2	 	   
	   read(21,*) tempword   
	   read(21,*) segs	   
	   read(21,*) tempword   
	   read(21,*) stepsize
	   read(21,*) tempword   
	   read(21,*) l_cut
	   read(21,*) tempword   !!!!!
	   read(21,*) first	     !!!!!
	   read(21,*) tempword   !!!!!
	   read(21,*) last   	 !!!!! 	   
	   read(21,*) tempword   
	   read(21,5) inline2
	   
 10     format(A,A)
	    write(6,10)'I am going to look for data in the file labeled: ', keyword
 11     format(A,I2,2X,A,I2,2X,A,I2,2X,A,F5.2)
	    write(6,11)'msym =',msym,'rotation =',rotation,'radian =',radian,'step size =',stepsize
 12     format(A,I2,2X,A,I2)
	    write(6,12)'first =',first,'last =',last	   
	    close(21)
		
c  The statement calls the subroutine that contains the symmetry operators.  The variable
c  msym, read from the input file, specifies the symmetry (1 = tetragonal, 2 = hexagonal, 
c  3 = subic, 4 = trigonal), O is a matrix containing the operators, and nsymm is the 
c  number of symmetry operators.  The routine returns O and nsymm.

       call get_symop (msym, O, nsymm)

c  This part is to zero the grain boundary character distribution matrix		
		do i=1,CD
	     do j=1,CD
	      do k=1,CD
	       do ii=1,CD2
	        do jj=1,4*CD2
	         gbd(i,j,k,ii,jj)=0.0
			enddo
	       enddo
		  enddo
	     enddo
	    enddo
		
		open (38, file='mss_cor_1_'//keyword//'.txt'  !!!!!!!
     &         ,status='old')                         !!!!!!!


c  this part measures the header of the correction file	   

       header = 0
       hash = 0
	   do i=1,500
	    read(38, "(a)") inline
	    hash = index(inline, '#')
	    if (hash.ne.0) then
	     header=header+1
		 hash=0
	    else
		 goto 50     
	    endif			
	   enddo
 50   close(38)		

c  here we read the correction data into the cor matrix	 
      !write(6,*)header
		open (38, file='mss_cor_1_'//keyword//'.txt'  !!!!!!!
     &         ,status='old')                         !!!!!!!
         do i=1,header
	      read(38, "(a)") inline
	     enddo			 
c  
		do i=1,CD
		 do j=1,CD
	      do k=1,CD
	       do ii=1,CD2
	        do jj=1,4*CD2
	         read (38,*) cor(i,j,k,ii,jj)
			enddo
	       enddo
		  enddo
	     enddo
	    enddo
		close(38)
	   
       do nfile=first,last	   !!!!!!the loop over files starts here

	    one=char(48+nfile)                                  !!!!!!!
		
c  This part is to determines the total number of line segments (lines in the data file),
c  assigned to the variable, nnline
		
		nnline = 0
		open (41, file=keyword//'_00'//one//'.txt'  !!!!!!!
     &         ,status='old')                       !!!!!!!
 60     continue
		read(41,*,end=61)
		nnline=nnline+1
		goto 60
 61     continue
        close(41)
		
c Check each line for the string '#'. If found, we know this line is part  
c of the header.  If not, we have reached the data section.		
		open (41, file=keyword//'_00'//one//'.txt'  !!!!!!!
     &         ,status='old')                       !!!!!!!
       header = 0
       hash = 0
	   do i=1,nnline
	    read(41, "(a)") inline
	    hash = index(inline, '#')
	    if (hash.ne.0) then
	     header=header+1
		 hash=0
	    else
		 goto 80     
	    endif			
	   enddo
 80   close(41)		
		

 102	format(A,I7,A)
 103	format(A,I2,A) 
		write(6,102) 'There are',nnline-header,' lines in the data file.'
		write(6,103) 'The calculation will use a resolution of ',CD,' bins per 90 degrees.'

c       here, we figure out the average length of the segments
c       this is one figure of merit for the reliability of the 
c       results

		open (41, file=keyword//'_00'//one//'.txt'  !!!!!!!
     &         ,status='old')                       !!!!!!!
		 
c       read the header
         do i=1,header
	      read(41, "(a)") inline
	     enddo			 
		 
		 l_tot=0.0	
		 t_out=0
		
 110     continue
		 read(41,*,end=120) e1(1), e1(2), e1(3), e2(1), e2(2), e2(3), 
     &	 temp, temp, x1, y1, x2, y2

         nl(1)=x2-x1
		 nl(2)=y2-y1
	     length=sqrt((nl(1)*nl(1))+(nl(2)*nl(2)))
		 steps=nint(length/stepsize)
		 l_steps=length/stepsize
		 l_tot=l_tot+l_steps
		 t_out=t_out+1
		 goto 110
 120     continue		 
		 close(41)
		 l_ave=l_tot/float(t_out)
		
         write(6,206)'on average, there are ',l_ave,' steps per segment'

c  Depending on how this parameter is set in the input file, the 
c  segment analysis can be skipped

        if (segs.ne.0) then

c  This section analyzes the distribution of line lengths	

		do i = 1,200
		 ls(i) = 0
		end do
		l_tot=0.0
		t_out=0
		
		open (41, file=keyword//'_00'//one//'.txt'  !!!!!!!
     &         ,status='old')                       !!!!!!!


c  read the header
        do i=1,header
	     read(41, "(a)") inline
	    enddo
		
 200    continue
		 read(41,*,end=201) e1(1), e1(2), e1(3), e2(1), e2(2), e2(3), 
     &	 temp, temp, x1, y1, x2, y2

         nl(1)=x2-x1
		 nl(2)=y2-y1
	     length=sqrt((nl(1)*nl(1))+(nl(2)*nl(2)))
		 steps=nint(length/stepsize)
		 l_steps=length/stepsize
		 
		 
		 if (l_steps.lt.l_cut) then
		  t_out=t_out+1
		 endif
		 
		 if (steps.eq.0) then
		  steps=1
		 endif

		 if (steps.gt.200) then
		  steps=200
		 endif
		 
		 ls(steps) = ls(steps)+1
		 l_tot=l_tot+l_steps
		
		 goto 200
 201    continue		 
		close(41)

        l_ave=l_tot/float(nnline)
		
 204    format(A,I9)		
 205	format(f5.1,A)	
 206    format(A,F5.1,A)
 207    format(A,F4.1,A,I5,A)
 
        smalls=100*(float(ls(1)+ls(2)+ls(3)+ls(4)+ls(5)+
     &  ls(6)+ls(7))/float(nnline))
        write(6,205) smalls, '% of the segments are less than 7.5 steps'
        smalls=100*(float(ls(1)+ls(2)+ls(3)+ls(4)+ls(5)+
     &  ls(6))/float(nnline))
        write(6,205) smalls, '% of the segments are less than 6.5 steps'
		smalls=100*(float(ls(1)+ls(2)+ls(3)+ls(4)+ls(5))/float(nnline))
        write(6,205) smalls, '% of the segments are less than 5.5 steps'
		smalls=100*(float(ls(1)+ls(2)+ls(3)+ls(4))/float(nnline))
        write(6,205) smalls, '% of the segments are less than 4.5 steps'
		smalls=100*(float(ls(1)+ls(2)+ls(3))/float(nnline))
        write(6,205) smalls, '% of the segments are less than 3.5 steps'
        smalls=100*(float(ls(1)+ls(2))/float(nnline))
        write(6,205) smalls, '% of the segments are less than 2.5 steps'
        smalls=100*(float(ls(1))/float(nnline))
        write(6,205) smalls, '% of the segments are less than 1.5 steps'

        write(6,206)'on average, there are ',l_ave,' steps per segment'
        write(6,207)'with a cut-off of',l_cut,', ',t_out,' segments 
     &  will be ignored because they are too small.'
		smalls=100*(float(t_out)/float(nnline))
        write(6,205) smalls, '% of the segments will be ignored.'
		
		
		open(25, file='segment_stats_'//keyword, status='unknown')
		write(25,102) 'There are',nnline,' lines in the data file.'
		write(25,206)'on average, there are ',l_ave,' steps per segment'
        smalls=100*(float(ls(1)+ls(2)+ls(3)+ls(4)+ls(5)+
     &  ls(6)+ls(7))/float(nnline))
        write(25,205)smalls, '% of the segments are less than 7.5 steps'
        smalls=100*(float(ls(1)+ls(2)+ls(3)+ls(4)+ls(5)+
     &  ls(6))/float(nnline))
        write(25,205)smalls, '% of the segments are less than 6.5 steps'
		smalls=100*(float(ls(1)+ls(2)+ls(3)+ls(4)+ls(5))/float(nnline))
        write(25,205)smalls, '% of the segments are less than 5.5 steps'
		smalls=100*(float(ls(1)+ls(2)+ls(3)+ls(4))/float(nnline))
        write(25,205)smalls, '% of the segments are less than 4.5 steps'
		smalls=100*(float(ls(1)+ls(2)+ls(3))/float(nnline))
        write(25,205)smalls, '% of the segments are less than 3.5 steps'
	    smalls=100*(float(ls(1)+ls(2))/float(nnline))
        write(25,205)smalls, '% of the segments are less than 2.5 steps'
        smalls=100*(float(ls(1))/float(nnline))
        write(25,205)smalls, '% of the segments are less than 1.5 steps'
        write(25,207)'with a cut-off of',l_cut,', ',t_out,' segments 
     &  will be ignored because they are too small.'
        smalls=100*(float(t_out)/float(nnline))
        write(25,205)smalls, '% of the segments will be ignored.'	
		

		
c       here, we figure out the average length of the remaining segments

		open (41, file=keyword//'_00'//one//'.txt'  !!!!!!!
     &         ,status='old')                       !!!!!!!
		 
c  read the header
        do i=1,header
	     read(41, "(a)") inline
	    enddo			 
		 
		 l_tot=0.0	
		 t_out=0
		
 300    continue
		 read(41,*,end=301) e1(1), e1(2), e1(3), e2(1), e2(2), e2(3), 
     &	 temp, temp, x1, y1, x2, y2

         nl(1)=x2-x1
		 nl(2)=y2-y1
	     length=sqrt((nl(1)*nl(1))+(nl(2)*nl(2)))
		 steps=nint(length/stepsize)
		 l_steps=length/stepsize
		 
		 if (l_steps.ge.l_cut) then
			l_tot=l_tot+l_steps
			t_out=t_out+1
		 endif	
		
		 goto 300
 301    continue		 
		close(41)

		 l_ave=l_tot/float(t_out)
		
        write(25,206)'on average, there are ',l_ave,' steps per segment
     &  for the segments included'
        write(6,206)'on average, there are ',l_ave,' steps per segment 
     &  for the segments included'
		
		
		write(25,*)' '
        write(25,*)'the following is a histogram of the number of segments 
     &  with a certain step length'
		write(25,*) 'length, steps    number'
		do i = 1,200
		 write(25,*)i, ls(i)
		end do

        close(25)

        endif  ! this ends the segment analysis branch

	    ref(1)=1.0
	    ref(2)=0.0
		mag_ref=1.0


		t_out=0.0
		open (41, file=keyword//'_00'//one//'.txt'  !!!!!!!
     &         ,status='old')                       !!!!!!!
		
c  read the header
        do i=1,header
	     read(41, "(a)") inline
	    enddo		
		
		nline = 0
		
 310    continue
        nline=nline+1		

c  This section reads a single line from the data file
		
		read(41,*,end=400) e1(1), e1(2), e1(3), e2(1), e2(2), e2(3), 
     &	temp, temp, x1, y1, x2, y2
	
c  This part converts angular data to radians, if specified in the input file
c  by the variable, radian
				
	    if (radian.eq.0) then
	     e1(1)=e1(1)*(pi/180.0)
	     e1(2)=e1(2)*(pi/180.0)
	     e1(3)=e1(3)*(pi/180.0)

	     e2(1)=e2(1)*(pi/180.0)
	     e2(2)=e2(2)*(pi/180.0)
	     e2(3)=e2(3)*(pi/180.0)
	    endif

c  If the data are in the HKL reference frame, and the system is hexagonal,
c  then the data are rotated by -30 so that [2-1-10] is parallel to +x in 
c  the output (this direction is to the right in the plane of the paper)
		
	    if (rotation.eq.0.and.msym.eq.2) then
	     if (e1(3).gt.(30*pi/180)) then
 	      e1(3) = e1(3)-(30*pi/180)
		 else
		  e1(3) = e1(3)+(330*pi/180)
		 endif
		 if (e2(3).gt.(30*pi/180)) then
 	      e2(3) = e2(3)-(30*pi/180)
		 else
		  e2(3) = e2(3)+(330*pi/180)
		 endif
		endif
		
c  This part converts the Euler angles from the input file to a g matrix

        call EToG(e1,g1)
		call EToG(e2,g2)

c This section determines the in plane components of the grain boundary segment,
c based on the endpoints in the data file.  The first calculation of nl(i) 
c transforms line segments into the same reference frame as the Euler angles,  
c based on and assumption of default settings in TSL or HKL software.  The second 
c calculation determines the normal to the grain boundary trace.  

	    if (rotation.eq.1) then       !TSL
	       nl(1)=y1-y2
	       nl(2)=x1-x2
		endif   
	    if (rotation.eq.0) then
	       nl(1)=x2-x1
	       nl(2)=-(y2-y1)
		endif
	    if (rotation.eq.2) then
	       nl(1)=x2-x1
	       nl(2)=y2-y1
		endif		
		if (rotation.eq.3) then
	       nl(1)=y2-y1
	       nl(2)=-(x2-x1)
		 endif

		if (rotation.eq.4) then ! put here for sim_obs version 05/11/2010
	       nl(1)=y2             ! because sim_obs writes normals to the 
	       nl(2)=-x2            ! traces instead of the trace
		 endif



	    templ=nl(2)                   !normal to the GB trace
  	    nl(2)=nl(1)
	    nl(1)=-templ
	    nl(3)=0.0
		
		
		!if (nline.eq.50) goto 9000  !diagnostic
		
	    length=sqrt(nl(1)*nl(1)+nl(2)*nl(2))
		steps=nint(length/stepsize)
        l_steps=length/stepsize
		 
		 if (l_steps.lt.l_cut) then
		  t_out=t_out+1
		  goto 390
		 endif
		
		!This section calculates phi for the GB normal, with respect to [100]
		mag_dir=((nl(1)**2)+(nl(2)**2))**0.5
		dot = nl(1)*ref(1) + nl(2)*ref(2)
	    p = acos(dot/(mag_dir*mag_ref))
		if (nl(2).lt.0.0) p=-p  !vectors pointing below the x-z plane assigned 
								! negative angles (acos make them positive) so
								! the range of phi is 0 - 180 for a positive y
								! coorinate and 0 - -180 for a negative y coord
		
	    !if (nl(1).ne.0.0) then  !not sure why this was originally in code?
	    !   p=atan(nl(2)/nl(1))
	    !else
	    !   p=pi*0.5
	    !endif
		
		!templ=p !diagnostic
	    !p=atan2(nl(2),nl(1)) ! this atan2 apparently did exactly the same as the
		                      ! as the four lines obaove to determine phi, but I 
							  ! wasn't sure about it.
		!write(6,*)nl(1), nl(2), templ*(180/pi), p*(180/pi)!diagnostic
		
	    if (p.lt.0.0) p=pi+p  ! For all directions with negative y (negative angle)
							  ! we consider the opposite direction
		
        do i=1,nsymm
          do j=1,nsymm
c         this call returns the i-th 3x3 symmetry operator from O		
		  call symop(O,i,so1)
c         this hits the first g matrix with the i-th symmetry operator		  
		  call MToM (so1,g1,gg1)
c		  this call returns the j-th 3x3 symmetry operator from O
		  call symop(O,j,so2)
c		  this hits the second g matrix with the j-th symmetry operator		   
		  call MToM(so2,g2,gg2)
c		  this takes the transpose of the second g matrix	
		  call trans(gg2,gt)
c		  this computes the misorientation between g1 and g2		   
		  call MToM(gg1,gt,dg)
c		  this returns three Euler angles for the misorientation across this segment
		  call GToE(dg,mort)
c         this condition considers only misorientations when all 3 angles are less than 90.
c         if so, the three misorientation indices in the GBCD matrix are determined (c1,c2,c3)		  
		  if (mort(1).lt.(pi*0.5+eps) .and. 
     &                       mort(2).lt.(pi*0.5+eps) .and. 
     &                       mort(3).lt.(pi*0.5+eps) ) then
     		c1=int(float(CD)*2.0*mort(1)/pi)+1
			c2=int(float(CD)*cos(mort(2)))+1
			c3=int(float(CD)*2.0*mort(3)/pi)+1
			if (c1.eq.CD+1) c1=CD
			if (c2.eq.CD+1) c2=CD
			if (c3.eq.CD+1) c3=CD
c           in the section that follows, we determine the orientation of the grian
c           along the sample normal, and find the appropriate scaling factor to apply
			nsam(1) = 0.0
            nsam(2) = 0.0
            nsam(3) = 1.0
			call MToV (gg1, nsam, nm)	
			call VToAngles (nm, sa)                           
			c4=int(float(CD2)*cos(sa(1)))+1
			c5=int(float(4*CD2)*sa(2)/(pi*2.0))+1		  
			if (c4.eq.CD2+1) c4=CD2
			if (c4.eq.0) c4=1
			if (c5.eq.4*CD2+1) c5=4*CD2
            tex_scale=cor(c1,c2,c3,c4,c5) 
	
c           the section puts the line segments into bins cooresponding to orientations normal
c           to the trace it does this in one degree increments, from 1 to 90.
			do k=1,int(HCD)
			  t=acos2((2.0*(float(k)-0.5)/HCD)-1.0)
			  call AnglesToV (t, p, n)
			  call MToV (gg1, n, nf)
			  call VToAngles (nf, sa)
	          c4=int(float(CD2)*cos(sa(1)))+1
			  c5=int(float(4*CD2)*sa(2)/(pi*2.0))+1		  
			  if (c4.eq.CD2+1) c4=CD2
			  if (c4.eq.0) c4=1
			  if (c5.eq.4*CD2+1) c5=4*CD2
			  gbd(c1,c2,c3,c4,c5)=gbd(c1,c2,c3,c4,c5)+(length*tex_scale)
			enddo !k-loop
		  endif
c		  this takes the transpose of the first g matrix			
		  call trans (gg1, gt)
c		  this computes the misorientation between g2 and g1		   
		  call MToM (gg2, gt, dg)
c		  this returns three Euler angles for the misorientation across this segment		   
		  call GToE (dg, mort)
c         this repeats the operations above for the interchanged bicrystal		
		  if (mort(1).lt.(pi*0.5+eps) .and. 
     &                       mort(2).lt.(pi*0.5+eps) .and. 
     &                       mort(3).lt.(pi*0.5+eps) ) then
     		c1=int(float(CD)*2.0*mort(1)/pi)+1
			c2=int(float(CD)*cos(mort(2)))+1
			c3=int(float(CD)*2.0*mort(3)/pi)+1
			if (c1.eq.CD+1) c1=CD
			if (c2.eq.CD+1) c2=CD
			if (c3.eq.CD+1) c3=CD
			nsam(1) = 0.0
            nsam(2) = 0.0
            nsam(3) = 1.0
			call MToV (gg2, nsam, nm)	
			call VToAngles (nm, sa)                           
			c4=int(float(CD2)*cos(sa(1)))+1
			c5=int(float(4*CD2)*sa(2)/(pi*2.0))+1		  
			if (c4.eq.CD2+1) c4=CD2
			if (c4.eq.0) c4=1
			if (c5.eq.4*CD2+1) c5=4*CD2
            tex_scale=cor(c1,c2,c3,c4,c5) 
			
			do k=1,int(HCD)
			  t=acos2((2.0*(float(k)-0.5)/HCD)-1.0)
			  call AnglesToV (t, p, n)
			  call MToV (gg2, n, nf)
			  call VToAngles (nf, sa)
			  c4=int(float(CD2)*cos(sa(1)))+1
			  c5=int(float(4*CD2)*sa(2)/(pi*2.0))+1		  
			  if (c4.eq.CD2+1) c4=CD2
			  if (c4.eq.0) c4=1
			  if (c5.eq.4*CD2+1) c5=4*CD2
			  gbd(c1,c2,c3,c4,c5)=gbd(c1,c2,c3,c4,c5)+(length*tex_scale)		  
			enddo ! k-loop
		  endif
		  enddo ! j-loop
        enddo ! i-loop

 390    continue

c  this just notifies you of the progress of the calculation
 
		if (mod(nline,5000).eq.0) then
	    write(6,204) 'Lines analyzed so far: ', nline
	    endif

	    goto 310
		
c  the program comes to line 200 when it reaches the last line of the data file.		
 400    continue		
        enddo   !!!!!!!!!!!!! end of the file loop 
		
		
c  this loop determines the average line length in each bin		
	    avg=0.0
	    do i=1,CD
	     do j=1,CD
	      do k=1,CD
	       do ii=1,CD2
	        do jj=1,4*CD2
	         avg=avg+gbd(i,j,k,ii,jj)
	        enddo
		   enddo
	      enddo
	     enddo
	    enddo
        
		avg=avg/float(4*CD*CD*CD*CD2*CD2)
		
c  this is the section where the background is subtracted

	   pnc=(2.0*float(CD2)-1.0)/(2.0*float(CD2))
	   f=(1.0/float(CD2))*(pnc/(1.0-pnc))

		do i=1,CD
	    do j=1,CD
	     do k=1,CD
	      avg=0.0
	      do ii=1,CD2
	       do jj=1,4*CD2
	       avg=avg+gbd(i,j,k,ii,jj)
	      enddo
	     enddo
	     avg=avg/float(4*CD2*CD2)
	     do ii=1,CD2
	      do jj=1,4*CD2
	       temp=gbd(i,j,k,ii,jj)
	       temp=(temp-avg*(pnc-f+f*pnc))/(1.0+f)
	       if (temp.gt.0.0) then
	        gbd(i,j,k,ii,jj)=temp
	       else
	        gbd(i,j,k,ii,jj)=0.0
	       endif
	      enddo
	     enddo
	    enddo
       enddo
	   enddo

c find the average value of the distribution
	    avg=0.0
	     do i=1,CD
	      do j=1,CD
	       do k=1,CD
	        do ii=1,CD2
	         do jj=1,4*CD2
	          avg=avg+gbd(i,j,k,ii,jj)
	         enddo
	        enddo
	       enddo
	      enddo
	     enddo
		 
	     avg=avg/float(4*CD*CD*CD*CD2*CD2)
 410    format(A,A,A,A)
 411    format(A,A)
c  write out the MRD values in the file gbcd.txt
	   open (32, file='gbcd_mss_1_'//keyword//'.txt', status='unknown')
		
       write(32,411)'# gbcd file written by msws_1: ',version
       write(32,411)'# based on file: ',keyword		
	   write(32,411)'# comment: ',inline2	
		
	    do i=1,CD
	     do j=1,CD
	      do k=1,CD
	       do ii=1,CD2
	        do jj=1,4*CD2
	         gbd(i,j,k,ii,jj)=gbd(i,j,k,ii,jj)/avg
	         write (32,*) gbd(i,j,k,ii,jj)
	        enddo
	       enddo
	      enddo
	     enddo
	    enddo
	    close (32)
		
		if (segs.ne.0) then
		write(6,102)'I threw away',t_out,' segments because they were too small. Therefore,'
		smalls=100*(float(t_out)/float(nnline))
		write(6,205) smalls, '% of your segments were ignored in the calculation'
		endif
		
 9000  continue  
       end